import 'package:flutter/material.dart';
import 'package:osam/domain/state/base_state.dart';
import 'package:osam/domain/usecase/usecase.dart';
import 'package:osam/osam.dart';
import 'package:osam/presentation/presenter.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

part '../../presentation/store_provider.dart';

class UseCaseNotFoundException implements Exception {
  final Type useCaseType;
  UseCaseNotFoundException(this.useCaseType);
}

class Store<ST extends BaseState> {
  final ST _state;
  final List<UseCase<ST>> useCases;

  U useCase<U extends UseCase<ST>>() {
    final useCase = _useCasesMap[U];
    if (useCase == null) {
      throw UseCaseNotFoundException(U);
    } else
      return useCase;
  }

  final _useCasesMap = <Type, UseCase<ST>>{};

  Store(this._state, {this.useCases}) {
    if (useCases?.isNotEmpty ?? false) {
      for (final useCase in useCases) {
        useCase.appState = _state;
        _useCasesMap[useCase.runtimeType] = useCase;
      }
    }
  }

  void _setStoreToUseCases(List<UseCase<ST>> newUseCases) =>
      newUseCases.forEach((useCase) => useCase.appState = _state);

  void updateUseCases(List<UseCase<ST>> newUseCases) {
    _setStoreToUseCases(newUseCases);
    newUseCases.forEach((useCase) => _useCasesMap[useCase.runtimeType] = useCase);
  }
}
